/****************************************************************************
*  This file is part of the speed test of ChibiOS/RT
*
*  Copyright (c) 2011-2013 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  20.02.2011  mifi   First Version, tested with a STM3210E-EVAL board
*  03.02.2013  mifi   Changed for CrossWorks version v2.3.0
****************************************************************************/
#define __MAIN_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include <ch.h>
#include <stm32f10x.h>
#include "nvic.h"

extern uint32_t SystemCoreClock;

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
/*
 * The LED1 is connected to port PC6.
 * If the LED on your eval board is connected to an 
 * other port, you must change the defines here and
 * the init code at LEDInit. 
 */
#define LED1      (1<<6)

#define LED_ON    GPIOC->BRR
#define LED_OFF   GPIOC->BSRR

/*
 * Priority and stack size of the task.
 */
#define TASK_HP_PRIORITY   NORMALPRIO + 3
#define TASK_LP_PRIORITY   NORMALPRIO + 2

#define TASK_HP_STK_SIZE   64
#define TASK_LP_STK_SIZE   64

#define AIRCR_VECTKEY      0x05FA0000
#define  SCB_AIRCR_PRIGROUP_0                ((uint32_t)0x00000100)        /*!< Bit 0 */
#define  SCB_AIRCR_PRIGROUP_1                ((uint32_t)0x00000200)        /*!< Bit 1 */
#define  SCB_AIRCR_PRIGROUP_2                ((uint32_t)0x00000400)        /*!< Bit 2  */
#define SYSCLK 72000000


/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static WORKING_AREA(HPStack, TASK_HP_STK_SIZE);
static WORKING_AREA(LPStack, TASK_LP_STK_SIZE);

static Thread* pHPThread;
static Thread* pLPThread;

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/
/***************************************************************************/
/*  LedOn                                                                  */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LedOn (void)
{
   LED_ON = LED1;
} /* LedOn */

/***************************************************************************/
/*  LedOff                                                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LedOff (void)
{
   LED_OFF = LED1;
} /* LedOff */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   /* Turn on GPIOC */
   RCC->APB2ENR = RCC_APB2ENR_IOPCEN;
   
   /* Set PC_6 to output */
   GPIOC->CRL &= ~GPIO_CRL_CNF6;
   GPIOC->CRL |= (GPIO_CRL_MODE6_0 | GPIO_CRL_MODE6_1);

   LedOff();
} /* LEDInit */

/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void HardwareInit (void)
{
   /* Disable the SysTick & using core source */
   SysTick->CTRL = SysTick_CTRL_CLKSOURCE;
   
   /* Reset the current value */
   SysTick->VAL &= ~SysTick_VAL_CURRENT;
   
   /* 
    * Setup the reload value
    *
    * To generate a multi-shot timer with a period of N processor 
    * clock cycles, use a RELOAD value of N-1. For example, if the 
    * SysTick interrupt is required every 100 clock pulses, set RELOAD to 99.
    */
   SysTick->LOAD = ((SystemCoreClock / CH_FREQUENCY) - 1L);
   
   /* Enable the SysTick */
   SysTick->CTRL = SysTick_CTRL_CLKSOURCE | SysTick_CTRL_ENABLE | SysTick_CTRL_TICKINT;
   
   /*
    * Init the GPIO for the LED
    */
   LEDInit();

} /* HardwareInit */

/***************************************************************************/
/*  HPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t HPTask (void *p)
{
   Thread *tp;
   msg_t Message;
   p = p;
   
   while(1)
   {
      tp = chMsgWait();
      Message = chMsgGet(tp);
      chMsgRelease(tp, Message);
      LedOff();
   }
} /* HPTask */

/***************************************************************************/
/*  LPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t LPTask (void *p)
{
   msg_t LPmessage = 42;
   
   p = p;
   
   while(1)
   {
      chThdSleepMilliseconds(100);
      
      LedOn();
      LedOff();
      
      LedOn();
      chMsgSend(pHPThread, 42);
   }
} /* LPTask */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  main                                                                   */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
int main (void)
{
   /*
    * Before the application which is using ChibiOS/RT can be started, 
    * the hardware of the target must be initialized. The CPU is still
    * initialized by the CrossWorks for ARM startup. But the timer tick
    * must be init too. Even the GPIO can be init here.
    */
   HardwareInit();

   /*
    * ChibiOS/RT initialization.
    */
   chSysInit();
      
   /*
    * Create our threads
    */
   pHPThread = chThdCreateStatic(HPStack, sizeof(HPStack), TASK_HP_PRIORITY, HPTask, NULL);
   pLPThread = chThdCreateStatic(LPStack, sizeof(LPStack), TASK_LP_PRIORITY, LPTask, NULL);
   
   /*
    * Normal main() thread activity.
    */
   while(1) 
   {
      chThdSleepMilliseconds(500);
   }

   /*
    * This return here make no sense. But to prevent the compiler warning:
    * "return type of 'main' is not 'int'
    * An int as return is used :-)
    */ 
   return(0);
} /* main */


/*** EOF ***/
